--	Code Doodle
--	by HugoBDesigner

function love.load()
	windowW, windowH = 600, 600
	score = -10 --Starts like this cause score is added on each new level
	love.window.setMode(windowW, windowH)
	love.graphics.setLineWidth(3)
	love.filesystem.setIdentity("spinning_ship")
	love.window.setTitle("Spinning Ship")
	font = love.graphics.newFont(windowW*.8)
	debugFont = love.graphics.newFont(12)
	scoreFont = love.graphics.newFont(windowW*.15)
	highFont = love.graphics.newFont(windowW*.03)
	
	math.randomseed(os.time())
	math.random(); math.random()
	
	debugMode = false
	
	failedGame = false
	high = 0
	level = 1
	maxSpeed = 1
	minSpeed = .5
	shipMaxSize = 15
	maxWheels = 5
	wheelDist = 50
	wheelWidth = 20
	speedOff = 1/20
	holePerc = .95
	
	newLevel()
end

function love.update(dt)
	for i, v in ipairs(wheels) do
		v.angle = v.angle + dt*v.speed*v.dir
		if v.angle < 0 then
			v.angle = v.angle+math.pi*2
		elseif v.angle >= math.pi*2 then
			v.angle = v.angle-math.pi*2
		end
	end
end

function love.draw()
	love.graphics.setFont(debugFont)
	love.graphics.setColor(255, 255, 255, 255)
	if debugMode then
		love.graphics.print("LEVEL: " .. level, 5, 5)
		love.graphics.print("SCORE: " .. score, 5, 15)
		love.graphics.print("HIGHSCORE: " .. high, 5, 25)
	end
	love.graphics.setFont(font)
	
	love.graphics.setColor(0, 55, 75, 255)
	local l = 16
	local d = 4
	for i = 1, l do
		love.graphics.print(level, windowW/2-font:getWidth(level)/2+ math.cos((i-1)/l*(math.pi*2))*d, windowH/2-font:getHeight()/2+ math.sin((i-1)/l*(math.pi*2))*d)
	end
	love.graphics.setColor(0, 105, 155, 255)
	love.graphics.print(level, windowW/2-font:getWidth(level)/2, windowH/2-font:getHeight()/2)
	
	love.graphics.setFont(scoreFont)
	love.graphics.setColor(55, 75, 0, 255)
	for i = 1, l do
		love.graphics.print(score, windowW-20-scoreFont:getWidth(score)+ math.cos((i-1)/l*(math.pi*2))*d, 5+ math.sin((i-1)/l*(math.pi*2))*d)
	end
	
	love.graphics.setColor(105, 155, 0, 255)
	love.graphics.print(score, windowW-20-scoreFont:getWidth(score), 5)
	
	love.graphics.setFont(highFont)
	
	d = d/2
	love.graphics.setColor(75, 55, 0, 255)
	for i = 1, l do
		love.graphics.print("High: " .. high, windowW-20-highFont:getWidth("High: " .. high)+ math.cos((i-1)/l*(math.pi*2))*d, 5+scoreFont:getHeight()+ math.sin((i-1)/l*(math.pi*2))*d)
	end
	
	love.graphics.setColor(155, 105, 0, 255)
	love.graphics.print("High: " .. high, windowW-20-highFont:getWidth("High: " .. high), 5+scoreFont:getHeight())
	
	love.graphics.push()
	love.graphics.translate(windowW/2, windowH/2)
	
	local function drawShip(r)
		love.graphics.setColor(55, 255, 0, 255)
		love.graphics.push()
		love.graphics.translate(r*wheelDist, 0)
		
		love.graphics.circle("fill", 0, 0, shipSize/2, 32)
		love.graphics.circle("line", 0, 0, shipSize/2, 32)
		love.graphics.polygon("fill", math.pi/4*shipSize/math.pi, -shipSize/2, math.pi/4*shipSize/math.pi, shipSize/2, shipSize, 0)
		if debugMode then
			love.graphics.line(0, 0, wheelDist*2, 0)
		end
		
		love.graphics.pop()
	end
	if shipWheel == 0 then
		love.graphics.push()
		love.graphics.rotate(-math.pi/2)
		drawShip(0)
		love.graphics.pop()
	end
	for i, v in ipairs(wheels) do
		if shipWheel >= i then
			love.graphics.setColor(0, 155, 255, 155)
		else
			love.graphics.setColor(255, 255, 255, 155)
		end
		if v.failed then
			love.graphics.setColor(255, 0, 0, 155)
		end
		love.graphics.push()
		love.graphics.rotate(v.angle)
		
		for a = 2, wheelFragments do
			local a1 = math.pi*2/wheelFragments*(a-1)
			local a2 = math.pi*2/wheelFragments*a
			
			love.graphics.polygon("fill", 	math.cos(a1)*(v.r-wheelWidth/2), math.sin(a1)*(v.r-wheelWidth/2),
											math.cos(a1)*(v.r+wheelWidth/2), math.sin(a1)*(v.r+wheelWidth/2),
											math.cos(a2)*(v.r+wheelWidth/2), math.sin(a2)*(v.r+wheelWidth/2),
											math.cos(a2)*(v.r-wheelWidth/2), math.sin(a2)*(v.r-wheelWidth/2))
			
			love.graphics.line(math.cos(a1)*(v.r+wheelWidth/2), math.sin(a1)*(v.r+wheelWidth/2), math.cos(a2)*(v.r+wheelWidth/2), math.sin(a2)*(v.r+wheelWidth/2))
			love.graphics.line(math.cos(a1)*(v.r-wheelWidth/2), math.sin(a1)*(v.r-wheelWidth/2), math.cos(a2)*(v.r-wheelWidth/2), math.sin(a2)*(v.r-wheelWidth/2))
		end
		
		local a1 = 0
		local a2 = math.pi*2/wheelFragments
		
		love.graphics.line(math.cos(a1)*(v.r-wheelWidth/2), math.sin(a1)*(v.r-wheelWidth/2), math.cos(a1)*(v.r+wheelWidth/2), math.sin(a1)*(v.r+wheelWidth/2))
		love.graphics.line(math.cos(a2)*(v.r+wheelWidth/2), math.sin(a2)*(v.r+wheelWidth/2), math.cos(a2)*(v.r-wheelWidth/2), math.sin(a2)*(v.r-wheelWidth/2))
		
		if debugMode then
			love.graphics.circle("line", 0, 0, v.r-wheelWidth/2, wheelFragments)
			love.graphics.circle("line", 0, 0, v.r+wheelWidth/2, wheelFragments)
			love.graphics.line(v.r, 0, math.cos(math.pi*2/wheelFragments)*v.r, math.sin(math.pi*2/wheelFragments)*v.r)
			
			if shipWheel+1 == i then
				love.graphics.line(0, 0, v.r, 0)
				love.graphics.line(0, 0, math.cos(math.pi/wheelFragments*2)*v.r, math.sin(math.pi/wheelFragments*2)*v.r)
			end
		end
		
		if shipWheel == i then
			love.graphics.rotate(math.pi/wheelFragments)
			drawShip(i)
		end
		
		love.graphics.pop()
	end
	
	love.graphics.pop()
end

function newLevel()
	love.graphics.setColor(255, 255, 255, 255)
	addScore(10)
	speed = minSpeed + (maxSpeed-minSpeed)/10*level
	
	shipWheel = 0
	shipSize = shipMaxSize-((level-1)/10*shipMaxSize/2)
	wheels = {}
	wheelFragments = 10+level
	
	for i = 1, maxWheels do
		local r = i*wheelDist
		local rot = math.random()*(math.pi*2)
		local dir = (math.random(2)-1)*2-1
		local speed = speed + (math.random()*2-1)*speedOff
		
		if i > 1 then
			while wheels[i-1].dir == dir and math.abs(wheels[i-1].speed-speed) <= minSpeed/2 do
				speed = speed + (math.random()*2-1)*speedOff
			end
		end
		
		table.insert(wheels, {r = r, dir = dir, speed = speed, angle = rot, failed = false})
	end
end

function love.mousepressed(x, y, button)
	if (button == "wu" or button == "wd") and debugMode then
		if button == "wu" then
			level = math.min(10, level+1)
		elseif button == "wd" then
			level = math.max(1, level-1)
		end
		newLevel()
	elseif button ~= "wu" and button ~= "wd" then
		move()
	end
end

function move()
	if shipWheel < maxWheels then
		local shipA = 3*math.pi/2
		if shipWheel > 0 then
			shipA = wheels[shipWheel].angle + math.pi/wheelFragments
		end
		local angleWindow = math.pi/wheelFragments*holePerc
		
		local nextA = wheels[shipWheel+1].angle + math.pi/wheelFragments
		
		if nextA + angleWindow >= math.pi*2 and shipA - angleWindow < 0 then
			shipA = shipA + math.pi*2
		end
		if shipA + angleWindow >= math.pi*2 and nextA - angleWindow < 0 then
			nextA = nextA + math.pi*2
		end
		
		if math.abs(shipA-nextA) <= angleWindow then
			--nothing
		else
			wheels[shipWheel+1].failed = true
			failedGame = true
			return
		end
		shipWheel = math.min(maxWheels, shipWheel+1)
		if not wheels[shipWheel].failed then
			addScore(1)
		end
	else
		if level == 10 and failedGame then
			level = 1
			score = 0
		else
			level = math.min(10, level+1)
		end
		newLevel()
	end
end

function addScore(n)
	score = score + n
	local h = 0
	if love.filesystem.exists("high.dat") then
		local n = love.filesystem.read("high.dat")
		if tonumber( tostring(n) ) then
			h = tonumber( tostring(n) )
		end
	end
	high = math.max(h, score)
	love.filesystem.write("high.dat", tostring(high))
end

function love.keypressed(key)
	if key == "d" and (love.keyboard.isDown("lctrl") or love.keyboard.isDown("rctrl")) then
		debugMode = not debugMode
	elseif key == "r" and (love.keyboard.isDown("lctrl") or love.keyboard.isDown("rctrl")) then
		if love.filesystem.exists("high.dat") then
			love.filesystem.remove("high.dat")
			high = 0
			addScore(0)
		end
	elseif key == "z" and (love.keyboard.isDown("lctrl") or love.keyboard.isDown("rctrl")) and debugMode then
		if shipWheel+1 <= maxWheels then
			wheels[shipWheel+1].failed = false
		end
	elseif key == "escape" then
		love.event.quit()
	elseif key ~= "lctrl" and key ~= "rctrl" then
		move()
	end
end